'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Updater = undefined;

var _sourceNode = require('./sourceNode');

var _utils = require('../utils');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Updater = exports.Updater = function () {
  function Updater(variables) {
    _classCallCheck(this, Updater);

    this.variables = variables.sort();
    this.name = (0, _utils.uniqueName)(this.variables);
    this.operators = [];
    this.complex = {};
    this.isCacheValue = false;
    this.declareVariableName = null;
  }

  _createClass(Updater, [{
    key: 'compile',
    value: function compile() {
      var _this = this;

      var sn = (0, _sourceNode.sourceNode)(null, 'function (__data__, ' + this.variables.join(', ') + ') {\n');

      if (this.isCacheValue) {
        sn.add('      __cache__.' + this.variables[0] + ' = ' + this.variables[0] + ';' + '\n');
      }

      if (this.declareVariableName) {
        sn.add('      var ' + this.declareVariableName + ';\n');
      }

      if (this.operators.length > 0) {
        sn.add((0, _sourceNode.sourceNode)(null, this.operators).join(';\n')).add(';\n');
      }

      if ((0, _utils.size)(this.complex) > 0) {
        var complexCode = [];

        Object.keys(this.complex).forEach(function (key) {
          var complex = _this.complex[key];

          var node = (0, _sourceNode.sourceNode)(null, '').add('      if (').add(complex.params.map(function (param) {
            return '__cache__.' + param + ' !== undefined';
          }).join(' && ')).add(') {\n').add(complex.compile()).add('\n      }');

          complexCode.push(node);
        });

        sn.add((0, _sourceNode.sourceNode)(null, complexCode).join('\n')).add('\n');
      }

      sn.add('    }');

      return sn;
    }
  }, {
    key: 'add',
    value: function add(code) {
      this.operators.push(code);
      return this;
    }
  }, {
    key: 'addComplex',
    value: function addComplex(loc, params, functionName) {
      params = params.sort();
      var complexName = (0, _utils.uniqueName)(params);

      if (!(complexName in this.complex)) {
        this.complex[complexName] = new Complex(params);
      }

      this.complex[complexName].add(functionName, [loc, params]);
      return this;
    }
  }, {
    key: 'cache',
    value: function cache() {
      if (this.variables.length == 1) {
        this.isCacheValue = true;
      } else {
        throw new Error('Value caching available only for setter with one variable.');
      }
      return this;
    }
  }, {
    key: 'declareVariable',
    value: function declareVariable(name) {
      this.declareVariableName = name;
      return this;
    }
  }]);

  return Updater;
}();

var Complex = function () {
  function Complex(params) {
    _classCallCheck(this, Complex);

    this.params = params;
    this.calls = [];
  }

  _createClass(Complex, [{
    key: 'add',
    value: function add(functionName, params) {
      this.calls[functionName] = params;
    }
  }, {
    key: 'compile',
    value: function compile() {
      var _this2 = this;

      var parts = [];

      Object.keys(this.calls).forEach(function (functionName) {
        var _calls$functionName = _slicedToArray(_this2.calls[functionName], 2);

        var loc = _calls$functionName[0];
        var params = _calls$functionName[1];

        parts.push((0, _sourceNode.sourceNode)(loc, ['        λ__', functionName, '(__data__, ', params.map(function (param) {
          return '__cache__.' + param;
        }).join(', '), ');']));
      });

      return (0, _sourceNode.sourceNode)(null, parts).join('\n');
    }
  }]);

  return Complex;
}();